function [x,c,A,b,lb,ub,csense,vtype,QCrows,QCvar1,QCvar2,QCcoef] = LMreadf(szInputFile)
% LMREADF: Read an LP/QP/MIP/MIQP problem in (extended) MPS or LINDO format, 
% and return the associated data objects characterizing the model. The input
% model is assumed to be in the following generic form.
%
%     optimize     f(x) = 0.5 x' Qc x + c' x 
%                         0.5 x' Qi x + A(i,:) x  ?  b(i)   for all i
%                      ub >=  x  >= lb
%                      x(v) is integer or binary
%
%     where,
%     Qc, and Qi are symmetric n by n matrices of constants for all i,
%     c, x and A(i,:) are n-vectors, and "?" is one of the relational 
%     operators "<=", "=", or ">=".
%
% Usage:  [x,c,A,b,lb,ub,csense,vtype,QCrows,QCvar1,QCvar2,QCcoef] = LMreadf(szInputFile)  
%
% Copyright (c) 2006
%
% LINDO Systems, Inc.            312.988.7422
% 1415 North Dayton St.          info@lindo.com
% Chicago, IL 60622              http://www.lindo.com          

%
% Last update Jan 09, 2007 (MKA)
%


global MY_LICENSE_FILE 
lindo;

if nargin<1
    help lmreadf
    return;
end;

c = []; A = []; b = [];
lb = []; ub=[]; csense=[];vtype=[];
QCrows=[];QCvar1=[];QCvar2=[];QCcoef=[];
x=[];
% Read license key from a license file
[MY_LICENSE_KEY,nErr] = mxlindo('LSloadLicenseString',MY_LICENSE_FILE);

% Create a LINDO environment
[iEnv,nErr]=mxlindo('LScreateEnv',MY_LICENSE_KEY);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

[iModel,nErr]=mxlindo('LScreateModel',iEnv);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;


[nErr]=mxlindo('LSreadMPSFile',iModel,szInputFile,LS_UNFORMATTED_MPS);
if nErr,
   [nErr]=mxlindo('LSreadMPSFile',iModel,szInputFile,LS_FORMATTED_MPS);
   if nErr,
      [nErr]=mxlindo('LSreadLINDOFile',iModel,szInputFile);
      if nErr,
          [nErr]=mxlindo('LSreadMPIFile',iModel,szInputFile);
          if nErr,
            fprintf('Bad input format or file does not exist.\n');
            return;
          end;         
      end;      
   end;
end;

%fprintf('LSreadXXXFile finished\n');

[osense,oshift,c,b,csense,A,lb,ub,nErr]=mxlindo('LSXgetLPData',iModel);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;
%fprintf('LSXgetLPData finished\n');
[QCrows,QCvar1,QCvar2,QCcoef,nErr]=mxlindo('LSgetQCData',iModel);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

%keep the dimension info in LHS vectors
[n, nErr] = mxlindo('LSgetInfo',iModel,LS_IINFO_NUM_VARS);
[m, nErr] = mxlindo('LSgetInfo',iModel,LS_IINFO_NUM_CONS);      
[nz, nErr] = mxlindo('LSgetInfo',iModel,LS_IINFO_NUM_NONZ);            
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

[vtype,nErr]=mxlindo('LSgetVarType',iModel);  
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

%[nErr]=mxlindo('LSwriteMPSFile',iModel,'temp.mps',1);
if 0,
   fprintf('\n%12s %16s %16s %16s\n','Variable','Lower Bound','Upper Bound','Cost');
   for k=0:n-1,
      [szVarname,nErr] = mxLINDO('LSgetVariableNamej',iModel,k);
      if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;
      fprintf('%12s %16.7e %16.7e %16.7e\n',szVarname,lb(k+1),ub(k+1),c(k+1));
   end;
   
   fprintf('\n%12s %16s %8s\n','Constraint','RHS','Sense');
   for k=0:m-1,
      [szConname,nErr] = mxLINDO('LSgetConstraintNamei',iModel,k);
      if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;
      fprintf('%12s %16.7e %8s\n',szConname,b(k+1),csense(k+1));
   end;
   
end;
      
    
% Un-hook
[nErr]=mxlindo('LSdeleteModel',iModel);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;

[nErr]=mxlindo('LSdeleteEnv',iEnv);
if nErr ~= LSERR_NO_ERROR, LMcheckError(iEnv,nErr) ; return; end;


